@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 1.5.5
+####  Version 1.6.0
 ####
 ####  Copyright 2012, University of Miami
 ####
@@ -84,6 +84,7 @@ rfsrc <- function(formula,
                   do.trace = FALSE,
                   membership = TRUE,
                   statistics = FALSE,
+                  fast.restore = FALSE,
                   ...)
 {
   user.option <- list(...)
@@ -139,6 +140,8 @@ rfsrc <- function(formula,
   xfactor <- extract.factor(data, xvar.names)
   yvar.types <- get.yvar.type(fmly, data[, yvar.names, drop = FALSE])
   data <- finalizeData(c(yvar.names, xvar.names), data, na.action, miss.flag)
+  data.row.names <- rownames(data)
+  data.col.names <- colnames(data)
   xvar <- as.matrix(data[, xvar.names])
   rownames(xvar) <- colnames(xvar) <- NULL
   split.wt <- NULL
@@ -146,8 +149,8 @@ rfsrc <- function(formula,
   n <- nrow(xvar)
   n.xvar <- ncol(xvar)
   mtry <- get.grow.mtry(mtry, n.xvar, fmly)
-  xvar.wt  <- get.grow.x.wt(xvar.wt, n.xvar)
-  split.wt <- get.grow.x.wt(split.wt, n.xvar)
+  xvar.wt  <- get.grow.xvar.wt(xvar.wt, n.xvar)
+  split.wt <- get.grow.split.wt(split.wt, n.xvar)
   forest.wt <- match.arg(as.character(forest.wt), c(FALSE, TRUE, "inbag", "oob", "all"))
   yvar <- as.matrix(data[, yvar.names, drop = FALSE])
   if(dim(yvar)[2] == 0) {
@@ -233,13 +236,14 @@ rfsrc <- function(formula,
   split.null.bits <- get.split.null(split.null)
   membership.bits <-  get.membership(membership)
   statistics.bits <- get.statistics(statistics)
+  fast.restore.bits <- get.fast.restore(fast.restore)
   # This is dependent on impute.only and the family being initialized.
   perf.flag <- get.perf(perf, impute.only, fmly)
   perf.bits <-  get.perf.bits(perf.flag)
   forest.wt.bits <- get.forest.wt(TRUE, bootstrap, forest.wt)
   na.action.bits <- get.na.action(na.action)
   do.trace <- get.trace(do.trace)
-  nativeOutput <- .Call("rfsrcGrow",
+  nativeOutput <- tryCatch({.Call("rfsrcGrow",
                         as.integer(do.trace),
                         as.integer(seed),
                         as.integer(impute.only.bits +
@@ -254,7 +258,8 @@ rfsrc <- function(formula,
                                    membership.bits +
                                    statistics.bits),
                         as.integer(0 +
-                                   na.action.bits),
+                                   na.action.bits +
+                                   fast.restore.bits),
                         as.integer(splitinfo$index),
                         as.integer(splitinfo$nsplit),
                         as.integer(mtry),
@@ -278,26 +283,30 @@ rfsrc <- function(formula,
                         as.double(event.info$time.interest),
                         as.double(miss.tree),
                         as.integer(nimpute),
-                        as.integer(get.rf.cores()))
-  if(is.null(nativeOutput)) {
-    stop("Error occurred in algorithm.  Please turn trace on for further analysis.")
+                        as.integer(get.rf.cores()))}, error = function(e) {NULL})
+  if (is.null(nativeOutput)) {
+    if (impute.only) {
+      return(NULL)
+    }
+    else {
+      stop("An error has occurred in the grow algorithm.  Please turn trace on for further analysis.")
+    }
   }
   if (n.miss > 0) {
     imputed.data <- matrix(nativeOutput$imputation, nrow = n.miss, byrow = FALSE)
     imputed.indv <- imputed.data[, 1]
-    imputed.data <- as.matrix(imputed.data[, -1])
-    if (n.miss == 1) imputed.data <- t(imputed.data)
+    imputed.data <- as.matrix(imputed.data[, -1, drop = FALSE])
     nativeOutput$imputation <- NULL
     if (nimpute > 1) {
       if (grepl("surv", fmly)) {
         yvar[imputed.indv, 1] <- imputed.data[, 1]
         yvar[imputed.indv, 2] <- imputed.data[, 2]
-        xvar[imputed.indv, ] <- imputed.data[, -c(1:2)]
+        xvar[imputed.indv, ] <- imputed.data[, -c(1:2), drop = FALSE]
       }
       else {
         if (!is.null(yvar.types)) {
-          yvar[imputed.indv, ] <- imputed.data[, length(yvar.types)]
-          xvar[imputed.indv, ] <- imputed.data[, -c(1:length(yvar.types))]
+          yvar[imputed.indv, ] <- imputed.data[, length(yvar.types), drop = FALSE]
+          xvar[imputed.indv, ] <- imputed.data[, -c(1:length(yvar.types)), drop = FALSE]
         }
         else {
           xvar[imputed.indv, ] <- imputed.data
@@ -306,6 +315,7 @@ rfsrc <- function(formula,
       imputed.indv <- NULL
       imputed.data <- NULL
       imputedOOBData <- NULL
+      na.action = "na.omit"
     }
     else {
       colnames(imputed.data) <- c(yvar.names, xvar.names)
@@ -313,6 +323,7 @@ rfsrc <- function(formula,
     }
   }
   xvar <- as.data.frame(xvar)
+  rownames(xvar) <- data.row.names
   colnames(xvar) <- xvar.names
   xvar <- map.factor(xvar, xfactor)
   if (fmly != "unsupv") {
@@ -338,6 +349,8 @@ rfsrc <- function(formula,
                                        nativeOutput$contPT,
                                        nativeOutput$mwcpSZ))
     names(nativeArray) <- c("treeID", "nodeID", "parmID", "contPT", "mwcpSZ")
+    nativeArrayTNDS <- list(nativeOutput$tnSURV, nativeOutput$tnMORT, nativeOutput$tnNLSN, nativeOutput$tnCSHZ, nativeOutput$tnCIFN, nativeOutput$tnREGR, nativeOutput$tnCLAS, nativeOutput$tnMCNT, nativeOutput$tnMEMB)
+    names(nativeArrayTNDS) <- c("tnSURV","tnMORT","tnNLSN","tnCSHZ","tnCIFN","tnREGR","tnCLAS","tnMCNT", "tnMEMB")
     nativeFactorArray <- nativeOutput$mwcpPT
     forest.out <- list(nativeArray = nativeArray,
                        nativeFactorArray = nativeFactorArray,
@@ -353,7 +366,11 @@ rfsrc <- function(formula,
                        xvar.names = xvar.names,
                        seed = nativeOutput$seed,
                        bootstrap.bits = bootstrap.bits,
-                       bootstrap = bootstrap)
+                       bootstrap = bootstrap,
+                       fast.restore.bits = fast.restore.bits,
+                       nativeArrayTNDS = nativeArrayTNDS,
+                       version = "1.6.0",
+                       na.action = na.action)
     if (grepl("surv", fmly)) {
       forest.out$time.interest <- event.info$time.interest
     }
@@ -381,9 +398,15 @@ rfsrc <- function(formula,
   }
   else {
     if (fmly == "class") {
-      err.names <- list(c("all", yfactor$levels[[1]]), NULL)
-      vimp.names <- list(c("all", yfactor$levels[[1]]), xvar.names)
-      ens.names <- list(NULL, yfactor$levels[[1]], NULL)
+      if (yfactor$generic.types == "I") {
+        ylevels <- yfactor$order.levels[[1]]
+      }
+      else {
+        ylevels <- yfactor$levels[[1]]
+      }
+      err.names <- list(c("all", ylevels), NULL)
+      vimp.names <- list(c("all", ylevels), xvar.names)
+      ens.names <- list(NULL, ylevels, NULL)
     }
     else {
       err.names <- list(NULL, NULL)
