@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.5.5
+////  Version 1.6.0
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -74,10 +74,17 @@ extern uint     *RF_mwcpPT_;
 extern double   *RF_spltST_;
 extern double   *RF_spltVR_;
 extern uint      RF_totalNodeCount;
+extern double   *RF_TN_SURV_;
+extern double   *RF_TN_MORT_;
+extern double   *RF_TN_NLSN_;
+extern double   *RF_TN_CSHZ_;
+extern double   *RF_TN_CIFN_;
+extern double   *RF_TN_REGR_;
+extern uint     *RF_TN_CLAS_;
+extern uint     *RF_TN_MCNT_;
+extern uint     *RF_TN_MEMB_;
 extern int      *RF_seed_;
 extern uint     *RF_tLeafCount_;
-extern double   *RF_proximity_;
-extern double   *RF_weight_;
 extern uint      RF_opt;
 extern uint      RF_optHigh;
 extern uint      RF_splitRule;
@@ -152,6 +159,9 @@ extern uint      RF_mrFactorSize;
 extern uint      RF_fmrFactorSize;
 extern uint     *RF_mrFactorIndex;
 extern uint     *RF_fmrFactorIndex;
+extern uint      RF_rNonFactorCount;
+extern uint     *RF_rNonFactorMap;
+extern uint     *RF_rNonFactorIndex;
 extern uint      RF_xFactorCount;
 extern uint     *RF_xFactorMap;
 extern uint     *RF_xFactorIndex;
@@ -160,6 +170,9 @@ extern uint      RF_mxFactorSize;
 extern uint      RF_fmxFactorSize;
 extern uint     *RF_mxFactorIndex;
 extern uint     *RF_fmxFactorIndex;
+extern uint      RF_xNonFactorCount;
+extern uint     *RF_xNonFactorMap;
+extern uint     *RF_xNonFactorIndex;
 extern uint      RF_rMaxFactorLevel;
 extern uint      RF_xMaxFactorLevel;
 extern uint      RF_maxFactorLevel;
@@ -187,13 +200,22 @@ extern double **RF_weightPtr;
 extern double   **RF_importancePtr;
 extern double **RF_sImputeResponsePtr;
 extern double **RF_sImputePredictorPtr;
-extern uint  **RF_tNodeMembershipIndexPtr;
+extern uint  **RF_tTermMembershipIndexPtr;
 extern uint  **RF_bootstrapMembershipPtr;
 extern uint  **RF_pNodeMembershipIndexPtr;
 extern double *RF_proximityDen;
 extern uint    RF_rejectedTreeCount;
 extern uint    RF_validTreeCount;
 extern uint    RF_stumpedTreeCount;
+extern double  ***RF_TN_SURV_ptr;
+extern double  ***RF_TN_MORT_ptr;
+extern double  ***RF_TN_NLSN_ptr;
+extern double ****RF_TN_CSHZ_ptr;
+extern double ****RF_TN_CIFN_ptr;
+extern double  ***RF_TN_REGR_ptr;
+extern uint   ****RF_TN_CLAS_ptr;
+extern uint     **RF_TN_MCNT_ptr;
+extern uint     **RF_TN_MEMB_ptr;
 extern double ***RF_oobEnsemblePtr;
 extern double ***RF_fullEnsemblePtr;
 extern double ***RF_oobCIFPtr;
@@ -214,9 +236,6 @@ extern double ***RF_splitDepthPtr;
 extern uint    *RF_serialTreeIndex;
 extern uint     RF_serialTreeCount;
 extern char    **RF_dmRecordBootFlag;
-extern double ***RF_dmvImputation;
-extern Terminal ***RF_mTermList;
-extern Terminal ***RF_mTermMembership;
 extern double **RF_performancePtr;
 extern uint   **RF_varUsedPtr;
 extern uint    *RF_oobSize;
@@ -230,15 +249,20 @@ extern Node    **RF_root;
 extern Node   ***RF_tNodeMembership;
 extern Node   ***RF_ftNodeMembership;
 extern Node   ***RF_pNodeMembership;
+extern Node   ***RF_tNodeList;
+extern Node   ***RF_pNodeList;
+extern Terminal   ***RF_tTermMembership;
+extern Terminal   ***RF_ftTermMembership;
+extern Terminal   ***RF_pTermMembership;
+extern Terminal   ***RF_tTermList;
+extern Terminal   ***RF_pTermList;
 extern uint    **RF_bootMembershipIndex;
 extern uint     *RF_identityMembershipIndex;
 extern char    **RF_bootMembershipFlag;
 extern uint    **RF_bootMembershipCount;
 extern char    **RF_oobMembershipFlag;
-extern Node   ***RF_tNodeList;
-extern Node   ***RF_pNodeList;
 extern uint     *RF_orderedLeafCount;
-extern Node  ****RF_vimpMembership;
+extern Terminal ****RF_vimpMembership;
 extern double  **RF_status;
 extern double  **RF_time;
 extern double ***RF_response;
