@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.5.5
+////  Version 1.6.0
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -68,6 +68,30 @@ struct terminal {
   unsigned int *lmiIndex;
   unsigned int  lmiSize;
   double       *lmiValue;
-  unsigned int  dominant;
+  unsigned int eTypeSize;
+  unsigned int mTimeSize;
+  unsigned int eTimeSize;
+  unsigned int sTimeSize;
+  unsigned int *atRiskCount;
+  unsigned int **eventCount;
+  unsigned int *eventTimeIndex;
+  double **localRatio;
+  double **localCSH;
+  double **localCIF;
+  double *localSurvival;
+  double *localNelsonAalen;
+  double **CSH;
+  double **CIF;
+  double *survival;
+  double *nelsonAalen;
+  double *mortality;
+  unsigned int   rnfCount;
+  double        *meanResponse;
+  unsigned int   rfCount;
+  unsigned int  *rfSize;
+  unsigned int **multiClassProb;
+  double predictedOutcome;
+  double weight;
+  unsigned int membrCount;
 };
 #endif
